<?php //$Id: backuplib.php,v 1.16 2009/09/10 17:26:03 juanca Exp $
/**
 * Backup All vpl module instances
 * @param $backupfile backup filedescriptor
 * @param $preferences backup preferences
 * @return bool (true==OK, false==Error)
 */

function vpl_backup_mods($backupfile,$preferences) {
	//This code was inspired in the backup of other moodle modules 
	global $CFG;
	require_once($CFG->dirroot.'/mod/vpl/fields_list.php');
	$status = true;
	//Get vpls of this course
	$vpls = get_records ("vpl","course",$preferences->backup_course,"id");
	if ($vpls) {
		foreach ($vpls as $vpl) {
			if (backup_mod_selected($preferences,'vpl',$vpl->id)) {
				$status = $status && assignment_backup_one_mod($backupfile,$preferences,$vpl);
			}
		}
	}
	return $status;
}

/**
 * Backup one vpl module instance
 * @param $backupfile backup filedescriptor
 * @param $preferences backup preferences
 * @param $vpl mix int id or object
 * @return bool (true==OK, false==Error)
 */
function vpl_backup_one_mod($backupfile,$preferences,$vpl) {
	//This code was inspired in the backuplib of other Moodle modules 
	debugging("Backingup vpl $vpl record");
	//Check if it is a single module backup
	if (is_numeric($vpl)) {
		$vpl = get_record('vpl','id',$vpl);
		if($vpl === false){
			return false;
		}
	}
	global $CFG;
	require_once($CFG->dirroot.'/mod/vpl/fields_list.php');
	@ob_flush();
	flush();
	
	$status = true;

	//Start mod
	fwrite ($backupfile,start_tag("MOD",3,true));
	//Backup register data
	fwrite ($backupfile,full_tag("ID",4,false,$vpl->id));
	fwrite ($backupfile,full_tag("MODTYPE",4,false,'vpl'));
	$fields = vpl_fields();
	foreach($fields as $field){
		$tag = strtoupper($field);
		fwrite ($backupfile,full_tag($tag,4,false,$vpl->$field));
	}

	$status = $status && vpl_backup_files($preferences,$vpl->id);
	//If requested backup submissions
	if (backup_userdata_selected($preferences,'vpl',$vpl->id)) {
		$status = $status && vpl_backup_submissions($backupfile,$preferences,$vpl->id);
	}
	$status =$status && fwrite ($backupfile,end_tag("MOD",3,true));
	return $status;
}

/**
 * Backup directory of confiration of vpl id
 * @param $backupfile back file name
 * @param $preferences preference of backup
 * @param $id vpl identification
 * @return bool (true==OK, false == error)
 */
function vpl_backup_files($preferences,$id) {
	//This code was inspired in the backup of assignment module
	global $CFG;
	debugging("Backingup vpl $id files");
	$status = true;

	//First we check to moddata exists and create it as necessary
	//in temp/backup/$backup_code  dir
	$status = check_and_create_moddata_dir($preferences->backup_unique_code);
	if ($status) {
		$dir_tail = '/'.$id.'/config';
		$from_dir = $CFG->dataroot.'/vpl_data/'.$preferences->backup_course.$dir_tail;
		$to_dir = $CFG->dataroot.'/temp/backup/'.$preferences->backup_unique_code.'/moddata/vpl'.$dir_tail;
		if (is_dir($from_dir)) {
			//Create destine directory
			$status = $status && check_dir_exists($to_dir,true,true);
			$status = $status && backup_copy_file($from_dir,$to_dir);
			//encode absolute links at fulldescription.html
			$fulldescription=$to_dir.'/fulldescription.html';
			if(file_exists($fulldescription)){
				$data = file_get_contents($fulldescription);
				$data = backup_encode_absolute_links($data);
				file_put_contents($fulldescription,$data);
			}			
		}
	}
	return $status;
}

/**
 * Backup All submissions registers and files
 * @param $backupfile
 * @param $preferences of backup
 * @param $vplid vpl identification
 * @return bool (true==OK, false == error)
 */
function vpl_backup_submissions($backupfile,$preferences,$vplid) {
	//This code was inspired in the backup of assignment module
	global $CFG;
	require_once($CFG->dirroot.'/mod/vpl/fields_list.php');
	
	debugging("Backingup vpl $vplid submissions files");
	$status = true;
	//Backup submissions files
	//First we check to moddata exists and create it as necessary
	//in temp/backup/$backup_code  dir
	$status = check_and_create_moddata_dir($preferences->backup_unique_code);
	if(!$status){
		return false;
	}
	$dir_tail = '/'.$vplid.'/usersdata/';
	$from_dir = $CFG->dataroot.'/vpl_data/'.$preferences->backup_course.$dir_tail;
	$to_dir = $CFG->dataroot.'/temp/backup/'.$preferences->backup_unique_code.'/moddata/vpl'.$dir_tail;

	//Backup submissions registers
	debugging("Backingup vpl $vplid submissions records");
	//Get last submissions records for this vpl module
	$tablename = $CFG->prefix.'vpl_submissions';
	$query = 'SELECT s.userid, s.* FROM '.$tablename.' s,';
	$query .= '(SELECT max(id) as maxid, userid, vpl FROM '.$tablename;
	$query .= ' WHERE vpl='.$vplid.' GROUP BY userid) ls';
	$query .= ' WHERE ls.maxid = s.id';
	$vpl_submissions = get_records_sql($query);

	if ($vpl_submissions !== false) {

		$status = $status && fwrite ($backupfile,start_tag("SUBMISSIONS",4,true));

		$fields = vpl_submission_fields();
		
		foreach ($vpl_submissions as $submission) {
			//Copy submission dir
			$subdir = $submission->userid.'/'.$submission->id;
			$status = $status && check_dir_exists($to_dir.$subdir,true,true);
			$status = $status && backup_copy_file($from_dir.$subdir,$to_dir.$subdir);
			
			//Save subsmission record
			$status =$status && fwrite ($backupfile,start_tag("SUBMISSION",5,true));
			fwrite ($backupfile,full_tag('ID',6,false,$submission->id));
			foreach($fields as $field){
				$tag = strtoupper($field);
				fwrite ($backupfile,full_tag($tag,6,false,$submission->$field));
			}

			$status =$status && fwrite ($backupfile,end_tag("SUBMISSION",5,true));
		}
		$status =$status && fwrite ($backupfile,end_tag("SUBMISSIONS",4,true));
	}

	return $status;
}

//Return an array of info (name,value)
function vpl_check_backup_mods($course,$user_data=false,$backup_unique_code,$instances=null) {
	if (!empty($instances) && is_array($instances) && count($instances)) {
		$info = array();
		foreach ($instances as $id => $instance) {
			$info += vpl_check_backup_mods_instances($instance,$backup_unique_code);
		}
		return $info;
	}
	//First the course data
	$info[0][0] = get_string('modulenameplural','vpl');
	$info[0][1] = vpl_count($course);

	//Now, if requested, the user_data
	if ($user_data) {
		$info[1][0] = get_string("submissions","vpl");
		$info[1][1] = vpl_submission_count($course);
	}
	return $info;
}

//Return an array of info (name,value)
function vpl_check_backup_mods_instances($instance,$backup_unique_code) {
	$info[$instance->id.'0'][0] = '<b>'.$instance->name.'</b>';
	$info[$instance->id.'0'][1] = '';
	if (!empty($instance->userdata)) {
		$info[$instance->id.'1'][0] = get_string("submissions","vpl");
		$info[$instance->id.'1'][1] = vpl_submission_count_by_instance($instance->id);
	}
	return $info;
}

//Return a content encoded to support interactivities linking. Every module
//should have its own. They are called automatically from the backup procedure.
function vpl_encode_content_links ($content,$preferences) {
	debugging("Enconding vpl links");
	global $CFG;

	$base = preg_quote($CFG->wwwroot,"/");

	//Link to the list of vpl
	$buscar="/(".$base."\/mod\/vpl\/index.php\?id\=)([0-9]+)/";
	$result= preg_replace($buscar,'$@VPLINDEX*$2@$',$content);

	//Link to vpl view by moduleid
	$buscar="/(".$base."\/mod\/vpl\/view.php\?id\=)([0-9]+)/";
	$result= preg_replace($buscar,'$@VPLVIEWBYID*$2@$',$result);

	return $result;
}

/**
 * return the number of vpls in course
 * @param $course course id
 * @return int
 */
function vpl_count($course) {
	$recs = get_records('vpl','course',$course,'id','id');
	if($recs === false){
		return 0;
	}else{
		return count($recs);
	}
}

/**
 * return the number of submissions in course
 * @param $course
 * @return int
 */
function vpl_submission_count($course) {

	global $CFG;
    $recs = get_records_sql ("SELECT s.id , s.vpl
                                 FROM {$CFG->prefix}vpl_submissions s,
                                 {$CFG->prefix}vpl a
                                 WHERE a.course = '$course' AND
                                       s.vpl = a.id");
    if($recs === false){
    	return 0;
    }else{
    	return count($recs);
    }           
}

/**
 * return the number of submissions by module
 * @param $course
 * @return int
 */
function vpl_submission_count_by_instance ($instanceid) {
	$recs = get_records('vpl_submissions','vpl',$instanceid,'id','id');
	if($recs === false){
		return 0;
	}else{
		return count($recs);
	}
}

?>