<?php
/**
 * @version		$Id: filegroup.class.php,v 1.7 2009/07/27 12:59:10 juanca Exp $
 * @package		vpl. class to edit a group of files
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino
 **/

class file_group_process{
	/**
	 * Name of file list
	 *
	 * @var string
	 */
	protected $filelistname;

	/**
	 * Path to directory where files are saved
	 *
	 * @var string
	 */
	protected $dir;

	/**
	 * Maximum number of files
	 *
	 * @var int
	 */
	protected $maxnumfiles;

	/**
	 * Number of files not changeables 0..($numstaticfiles-1)
	 *
	 * @var int
	 */
	protected $numstaticfiles;

	/**
	 * Constructor
	 *
	 * @param string $filelistname
	 * @param string $dir
	 * @param int $maxnumfiles
	 * @param int $numstaticfiles
	 */
	function __construct($filelistname,$dir,$maxnumfiles=10000,$numstaticfiles=0){
		$this->filelistname = $filelistname;
		$this->dir = $dir;
		$this->maxnumfiles = $maxnumfiles;
		$this->numstaticfiles = $numstaticfiles;
	}

	/**
	 * Get max number of files.
	 *
	 * @return int
	 */
	function get_maxnumfiles(){
		return $this->maxnumfiles;
	}

	/**
	 * Get number of static files.
	 *
	 * @return int
	 */
	function get_numstaticfiles(){
		return $this->numstaticfiles;
	}

	/**
	 * Add a new file to the group/Modify the data file
	 *
	 * @param string $filename
	 * @param string $data
	 * @return bool (added==true)
	 */
	function addFile($filename,$data=null){
		if($filename==''){
			return false;
		}
		$filelist = $this->getFileList();
		foreach($filelist as $f){
			if($filename == $f){
				if($data !== null){
					$fd = vpl_fopen($this->dir.$filename);
					fwrite($fd,$data);
					fclose($fd);
				}
				return true;
			}
		}
		if(count($filelist)>= $this->maxnumfiles){
			return false;
		}
		$filelist[] = $filename;
		$this->setFileList($filelist);
		if($data){
			$fd = vpl_fopen($this->dir.$filename);
			fwrite($fd,$data);
			fclose($fd);
		}
		return true;
	}

	/**
	 * Delete a file from groupfile
	 *
	 * @param int $num file position
	 * @return bool
	 */
	function deleteFile($num){
		if($num<$this->numstaticfiles){
			return false;
		}
		$filelist = $this->getFileList();
		$l = count($filelist);
		$ret = false;
		$filelistmod = array();
		for($i = 0 ;$i <$l; $i++){
			if($num== $i){
				$fullname = $this->dir.$filelist[$num];
				$ret = true;
				if(file_exists($fullname)){
					unlink($fullname);
				}
			}
			else{
				$filelistmod[]=$filelist[$i];
			}
		}
		if($ret){
			$this->setFileList($filelistmod);
		}
		return $ret;
	}

	/**
	 * Rename a file
	 *
	 * @param int $num
	 * @param string $filename new filename
	 * @return bool (renamed==true)
	 */
	function renameFile($num,$filename){
		if($num<$this->numstaticfiles || $filename == ''){
			return false;
		}
		$filelist = $this->getFileList();
		if(array_search($filename,$filelist) !== false){
			return false;
		}
		if($num<count($filelist)){
			if(file_exists($this->dir.$filelist[$num])){
				rename($this->dir.$filelist[$num],$this->dir.$filename);
			}
			$filelist[$num] =$filename;
			$this->setFileList($filelist);
		}
		return true;
	}

	/**
	 * Get list of files
	 *
	 * @return string[]
	 */
	function getFileList(){
		return vpl_read_list_from_file($this->filelistname);
	}

	/**
	 * Set the file list.
	 *
	 * @param string[] $filelist
	 */
	function setFileList($filelist){
			vpl_write_list_to_file($this->filelistname,$filelist);
	}
	
	/**
	 * Get the file comment by number
	 *
	 * @param int $num
	 * @return string
	 */
	function getFileComment($num){
		return get_string('file').' '.($num+1);
	}

	/**
	 * Get the file data by number o name
	 *
	 * @param int/string $mix
	 * @return string
	 */
	function getFileData($mix){
		if(is_int($mix)){
			$num=$mix;
			$filelist = $this->getFileList();
			if($num>=0 && $num<count($filelist)){
				$filename =$this->dir.$filelist[$num];
				if(file_exists($filename)){
					return file_get_contents($filename);
				}
			}
		}
		elseif(is_string($mix)){
			$filename=basename($mix);
			$filelist = $this->getFileList();
			if(array_search($filename,$filelist)!== false){
				$fullfilename =$this->dir.$filename;
				if(file_exists($fullfilename)){
					return file_get_contents($fullfilename);
				}
			}
		}
		return '';
	}
}
?>
