<?php
/**
 * @version		$Id: jail_server_mock.php,v 1.9 2009/08/09 13:20:38 juanca Exp $
 * @package		vpl. Jail Server Mock class
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino
 **/

/**
 * Establish a proxy service to conect client<=>jail o jail<=>client.
 * To create un object need a port range to select one.
 * Generate a password to acept a conection. The password must be recived first.
 * Main operations:
 *    advance(): to oerform comunication (non blocking)
 *    read(): read income information
 *    write(): send information (buffered)
 */
class vpl_mock_program{
	private $socket;
	private $state;
	private $towrite;
	private $connection_time;
	const   error=0;
	const   connected=1;
	const   closed=2;
	const   timeout=1; //1 sec
	/**
	 * Close sockects and set closed state
	 *
	 */
	function close(){
		socket_shutdown($this->socket,2);
		socket_close($this->socket);
		$this->status = self::closed;
	}

	/**
	 * Set de range of port to use
	 */
	function __construct($server,$port,$password){
		$this->towrite= $password;
		$this->socket=socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
		if (!is_resource($this->socket)) {
			$this->state = self::error;
			return;
		}
//		socket_set_nonblock($this->socket);
		for($i=0; $i<10; $i++){
        	$connected = socket_connect($this->socket, $server, $port);
        	if($connected){
        		break;
        	}
        	usleep(5000);
		}
        if (!$connected) {
            $this->state = self::error;
        }else{
        	$this->state = self::connected;
        }
	}

	/*
	 * Get object state (error=0, waiting=1, checking_password=2, connected=3, closed=4)
	 * @return int state
	 */
	function get_state(){
		return $this->state;
	}

	/*
	 * Read and send data if posible
	 */
	function advance(){
		if($this->state ==self::connected){
			if(socket_select($read=array($this->socket),$write =null,$except = null,0)){
				 if(count($read)){
					$data = socket_read($this->socket,1024);
					if($data !== false){
						$this->towrite .= $data;
					}else{
						$this->close();
						$this->state = self::close();
					}
				}
			}
			if( strlen($this->towrite) && socket_select($read=null,$write = array($this->socket),$except = null,0)){
				if(count($write) && strlen($this->towrite)){
					$writen= socket_write($this->socket,$this->towrite);
					if($writen === false){
						$this->close();
						$this->state = self::close();
					}else{
						if($writen>0){
							$this->towrite = substr($this->towrite,$writen);
						}
					}
				}
			}
		}
		return  $this->state;
	}
}


// PHP 5.2.2: $HTTP_RAW_POST_DATA not populated bug:
// http://bugs.php.net/bug.php?id=41293
if (empty($HTTP_RAW_POST_DATA)) {
	$HTTP_RAW_POST_DATA = file_get_contents('php://input');
}
$header_text='Content-type: text/xml; charset=utf-8';
$read = xmlrpc_decode_request($HTTP_RAW_POST_DATA,$method,'UTF-8');
if(xmlrpc_is_fault($read)){
	header($header_text);
	$read = $read['faultString'];
	echo xmlrpc_encode($read);
	die;
}else if($method == 'status'){
	header($header_text);
	$data = new stdClass();
	$data->status='ready';
	echo xmlrpc_encode($data);
	die;
}else if($method == 'execute'){
	//TODO implement console mock
	if(isset($_SERVER['HTTP_X_FORWARDED_FOR'])){
		$userip = $_SERVER['HTTP_X_FORWARDED_FOR'];
	}else{
		$userip = $_SERVER['REMOTE_ADDR'];
	}
	$data = new stdClass();
	$data->compilation = 'Sorry, fake compilation';//print_r($read,true);
	$data->executed='1';
    if(false && $read['interactive']){
		$toSend = xmlrpc_encode($data);
    	header($header_text);
		//header("Content-type: text/xml;charset=utf-8\nContent-length: ".strlen($toSend)."\n\n");
    	echo $toSend;
		flush();
		$now=time();
		$fakeprogram = new vpl_mock_program($userip,$read['port'],$read['password']."Hello from mock jail\n");
		while($fakeprogram->get_state() == vpl_mock_program::connected && (time()-$now)<6){
			usleep(10000);
			$fakeprogram->advance();
		}
	}else{
		$data->execution='Sorry I\'am a mock';
		$toSend = xmlrpc_encode($data);
    	header($header_text);
		//header("Content-type: text/xml;charset=utf-8\nContent-length: ".strlen($toSend)."\n\n");
    	print $toSend;
		flush();
	}
	die;
}
?>