<?php
/**
 * @version		$Id: sh_c.class.php,v 1.7 2009/07/27 12:59:10 juanca Exp $
 * @package		vpl. vpl Syntaxhighlighter for C language
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino
 **/

require_once('sh_base.class.php');

class vpl_sh_c extends vpl_sh_base{
	protected function show_pending(&$rest){
		if(array_key_exists($rest  , $this->reserved)){
			$this->initTag(self::c_reserved);
			parent::show_pending($rest);
			echo self::endTag;
		}else{
			parent::show_pending($rest);
		}
	}
	const regular=0;
	const in_string=1;
	const in_char=2;
	const in_macro=3;
	const in_comment=4;
	const in_linecomment=5;	
	function __construct(){
		$this->reserved= array("auto" => true,"break"=> true,"case"=> true,"char"=> true,
		                       "const"=> true,"continue"=> true,"default"=> true,"do"=> true,
		                       "double"=> true,"else"=> true,"enum"=> true,"extern"=> true,
		                       "float"=> true,"for"=> true,"goto"=> true,"if"=> true,
		                       "inline"=> true, "int"=> true, "long"=> true,"register"=> true,
		                       "restrict"=> true, "return"=> true,"short"=> true,"signed"=> true,
		                       "sizeof"=> true,"static"=> true,"struct"=> true,"switch"=> true,
                               "typedef"=> true,"union"=> true,"unsigned"=> true,"void"=> true,
                               "volatile"=> true,"while"=> true,"_Bool"=> true,
		                       "_Complex"=> true, "_Imaginary"=> true);
		parent::__construct();
	}
	function show_line_number(){
		echo "\n";
		parent::show_line_number();
	}
	

	function print_file($filename, $filedata){
		$this->begin($filename);
		$state = self::regular;
		$pending='';
		$first_no_space = '';
		$last_no_space = '';
		$l = strlen($filedata);
		if($l){
			$this->show_line_number();
		}
		$current='';
		$previous='';
		for($i=0;$i<$l;$i++){
			$previous=$current;
			$current=$filedata[$i];
			if($i < ($l-1)) {
				$next = $filedata[$i+1];
			}else{
				$next ='';
			}
			if($previous == self::LF){
				$last_no_space='';
				$first_no_space = '';
			}
			if($current == self::CR){
				if($next == self::LF) {
					continue;
				}else{
					$current = self::LF;
				}
			}
			if($current != ' ' && $current != "\t") {//Keep first and last char
				if($current != self::LF){
					$last_no_space=$current;
				}
				if($first_no_space == ''){
					$first_no_space = $current;
				}
			}
			switch($state){
				case self::in_comment:
					// Check end of block comment
					if($current=='*') {
						if($next=='/') {
							$state = self::regular;
							$pending .= '*/';
							$this->show_pending($pending);
							$this->endTag();
							$i++;
							continue;
						}
					}
					if($current == self::LF){
						$this->show_pending($pending);
						$this->endTag();
						$this->show_line_number();
						$this->initTag(self::c_comment);
					}else{
						$pending .= $current;
					}
					break;
				case self::in_linecomment:
					// Check end of comment
					if($current==self::LF){
						$this->show_pending($pending);
						$this->endTag();
						$this->show_line_number();
						$state=self::regular;
					}else{
						$pending .= $current;
					}
					break;
				case self::in_macro:
					// Check end of macro
					if($current==self::LF){
						if($last_no_space != '\\'){
							$this->show_pending($pending);
							$this->endTag();
							$this->show_line_number();
							$state = self::regular;
						}else{
							$this->show_pending($pending);
							$this->endTag();
							$this->show_line_number();
							$this->initTag(self::c_macro);
						}
					}else{
						$pending .= $current;
					}
					break;
				case self::in_string:
					// Check end of string
					if($current=='"' && $previous!='\\') {
						$pending .= '"';
						$this->show_pending($pending);
						$this->endTag();
						$state = self::regular;
						break;
					}
					if($current==self::LF){
						$this->show_pending($pending);
						$this->endTag();
						$this->show_line_number();
						$this->initTag(self::c_string);
					}else{
						$pending .= $current;
					}
					//discard two backslash
					if($current=='\\' && $previous=='\\'){
						$current=' ';
					}
					break;
				case self::in_char:
					// Check end of char
					if($current=='\'' && $previous!='\\') {
						$pending .= '\'';
						$this->show_pending($pending);
						$this->endTag();
						$state = self::regular;
						break;
					}
					if($current==self::LF){
						$this->show_pending($pending);
						$this->endTag();
						$this->show_line_number();
						$this->initTag(self::c_string);
					}else{
						$pending .= $current;
					}
					//discard two backslash
					if($current=='\\' && $previous=='\\'){
						$current=' ';
					}
					break;
				case self::regular:
					if($current == '/') {
						if($next == '*') { // Begin block comments
							$state = self::in_comment;
							$this->show_pending($pending);
							$this->initTag(self::c_comment);
							$this->show_text('/*');
							$i++;
							continue;
						}
						if($next == '/'){ // Begin line comment
							$state = self::in_linecomment;
							$this->show_pending($pending);
							$this->initTag(self::c_comment);
							$this->show_text('//');
							$i++;
							continue;
						}
					}elseif($current == '"')	{
						$state = self::in_string;
						$this->show_pending($pending);
						$this->initTag(self::c_string);
						$this->show_text('"');
						break;
					}elseif($current == "'"){
						$state = self::in_char;
						$this->show_pending($pending);
						$this->initTag(self::c_string);
						$this->show_text('\'');
						break;
					} elseif($current == '#' && $first_no_space==$current){
						$state = self::in_macro;
						$this->show_pending($pending);
						$this->initTag(self::c_macro);
						$this->show_text('#');
						break;
					}
					if(($current >= 'a' && $current <= 'z') ||
					($current >= 'A' && $current <= 'Z') ||
					($current >= '0' && $current <= '9') ||
					$current=='_' || ord($current) > 127){
						$pending .= $current;
					} else {
						$this->show_pending($pending);
						if($current == '}' || $current == '{'){
							$this->initTag(self::c_reserved);
							$this->show_pending($current);
							$this->endTag();
						}else{
							if($current == self::LF){
								$this->show_line_number();
							}else{
								$aux =$current;
								$this->show_pending($aux);
							}
						}
					}
			}
		}

		$this->show_pending($pending);
		if($state != self::regular){
			$this->endTag();
		}
		$this->end();
	}
}

?>