<?php
/**
 * @version		$Id: vpl.class.php,v 1.64 2009/10/27 16:32:33 juanca Exp $
 * @package		vpl. vpl class definition
 * @copyright	Copyright (C) 2009 Juan Carlos Rodríguez-del-Pino. All rights reserved.
 * @license		GNU/GPL, see LICENSE.txt or http://www.gnu.org/licenses/gpl-2.0.html
 * @author		Juan Carlos Rodriguez-del-Pino
 **/

/**
 * Module instance files
 * root_path= vpl_data/course_id#/vpl_instance#
 * General info
 * root_path/config/fulldescription.html
 * root_path/config/files_required.lst
 * root_path/config/initial_files/
 * root_path/config/execution_files/
 * root_path/config/execution_files/vpl_run.sh
 * root_path/config/execution_files/vpl_debug.sh
 * root_path/config/execution_files/vpl_evaluate.sh
 *  * Submission info
 * root_path/usersdata/userid#/submissionid#/submissionfiles/
 * root_path/usersdata/userid#/submissionid#/grade_comments.txt
 * root_path/usersdata/userid#/submissionid#/teachertest.txt
 * root_path/usersdata/userid#/submissionid#/studenttest.txt
 */

require_once('filegroup.class.php');

class file_group_execution extends file_group_process{
	/**
	 * Name of fixed file names
	 *
	 * @var string[]
	 */
	static protected $base_files = array('vpl_run.sh','vpl_debug.sh', 'vpl_evaluate.sh');

	/**
	 * Number of $base_files elements
	 *
	 * @var int
	 */
	static protected $num_base_files;

	/**
	 * Constructor
	 *
	 * @param string $filelistname
	 * @param string $dir
	 */
	function __construct($filelistname,$dir){
		self::$num_base_files=count(self::$base_files);
		parent::__construct($filelistname,$dir,1000,self::$num_base_files);
	}

	/**
	 * Get list of files
	 *
	 * @return string[]
	 */
	function getFileList(){
		return array_values(array_unique(array_merge(self::$base_files,parent::getFileList())));
	}

	/**
	 * Get the file comment by number
	 *
	 * @param int $num
	 * @return string
	 */
	function getFileComment($num){
		if($num<self::$num_base_files){
			return get_string(self::$base_files[$num],VPL);
		}else{
			return get_string('file').' '.($num+1-self::$num_base_files);
		}
	}

	/**
	 * Get list of files to keep when running
	 *
	 * @return string[]
	 */
	function getFileKeepList(){
		return vpl_read_list_from_file($this->filelistname.'.keep');
	}

	/**
	 * Set the file list to keep when running
	 *
	 * @param string[] $filelist
	 */
	function setFileKeepList($filelist){
		vpl_write_list_to_file($this->filelistname.'.keep',$filelist);
	}
}

class mod_vpl {
	/**
	 * Internal var for course_module
	 * @var object $cm
	 */
	protected $cm;

	/**
	 * Internal var for course
	 * @var object $course
	 */
	protected $course;

	/**
	 * Internal var for vpl
	 * @var object $instance
	 */
	protected $instance;

	/**
	 * Internal var object to requied file group manager
	 * @var object of file group manager
	 */
	protected $required_fgm;

	/**
	 * Internal var object to execution file group manager
	 * @var object of file group manager
	 */
	protected $execution_fgm;

	/**
	 * Constructor
	 * @param $id int optional course_module id
	 * @param $a int optional module instance id
	 */
	function __construct($id, $a=null) {
		if($id){
			if (! $this->cm = get_coursemodule_from_id(VPL,$id)) {
				print_error('invalidcoursemodule');
			}
			if (! $this->course = get_record("course", "id", $this->cm->course)) {
				print_error('unknowncourseidnumber','',$this->cm->course);
			}
			if (! $this->instance = get_record(VPL, "id", $this->cm->instance)) {
				print_error('module instance id unknow');
			}
		}else{
			if (! $this->instance = get_record(VPL, "id", $a)) {
				print_error('module instance id unknow');
			}
			if (! $this->course = get_record("course", "id", $this->instance->course)) {
				print_error('unknowncourseidnumber','',$this->instance->course);
			}
			if (! $this->cm = get_coursemodule_from_instance(VPL, $this->instance->id, $this->course->id)) {
				print_error('invalidcoursemodule');
			}
		}
		$this->required_fgm = null;
		$this->execution_fgm = null;
	}

	/**
	 * @return module db instance
	 **/
	function get_instance(){
		return $this->instance;
	}

	/**
	 * @return course
	 **/
	function get_course(){
		return $this->course;
	}

	/**
	 * @return course_module
	 **/
	function get_course_module(){
		return $this->cm;
	}

	/**
	 * Delete a vpl instance
	 *
	 * @return bool true if all OK
	 **/
	function delete_all(){
		//Delete all data files
		vpl_delete_dir($this->get_data_directory());
		//delete grade_item
		vpl_delete_grade_item($this->instance);
		//delete event
		delete_records('event', 'modulename', VPL, 'instance', $this->instance->id);
		//Delete all submissions records
		delete_records('vpl_submissions','vpl',$this->instance->id);
		//delete vpl record
		delete_records(VPL, 'id', $this->instance->id);
		// Allways true
		return true;
	}

	/**
	 * @return instance data directory
	 **/
	function get_data_directory(){
		global $CFG;
		return $CFG->dataroot.'/vpl_data/'.$this->course->id.'/'.$this->instance->id;
	}

	/**
	 * @return instance config data directory
	 **/
	function get_users_data_directory(){
		return $this->get_data_directory().'/usersdata';
	}

	/**
	 * @return instance users data directory
	 **/
	function get_config_data_directory(){
		return $this->get_data_directory().'/config';
	}
	
	/**
	 * @return fulldescription filename
	 **/
	function get_fulldescription_filename(){
		return $this->get_config_data_directory().'/fulldescription.html';
	}


	/**
	 * @return directory to stored initial required files
	 **/
	function get_required_files_directory(){
		return $this->get_config_data_directory().'/required_files/';
	}

	/**
	 * @return filename to store required files
	 **/
	function get_required_files_filename(){
		return $this->get_config_data_directory().'/required_files.lst';
	}


	/**
	 * @return array of files required name
	 **/
	function get_required_files(){
		return vpl_read_list_from_file($this->get_required_files_filename());
	}

	/**
	 * @param $files array of required files
	 **/
	function set_required_files($files){
		vpl_write_list_to_file($this->get_required_files_filename(),$files);
	}

	/**
	 * @return object file group manager for execution files
	 **/
	function get_required_fgm(){
		if(!$this->required_fgm){
			$this->required_fgm = new file_group_process($this->get_required_files_filename(),
			$this->get_required_files_directory(), $this->instance->maxfiles);
		}
		return $this->required_fgm;
	}

	/**
	 * @return directory to stored execution files
	 **/
	function get_execution_files_directory(){
		return $this->get_config_data_directory().'/execution_files/';
	}

	/**
	 * @return filename to store execution files
	 **/
	function get_execution_files_filename(){
		return $this->get_config_data_directory().'/execution_files.lst';
	}

	/**
	 * @return array of files execution name
	 **/
	function get_execution_files(){
		return vpl_read_list_from_file($this->get_execution_files_filename());
	}

	/**
	 * @return object file group manager for execution files
	 **/
	function get_execution_fgm(){
		if(!$this->execution_fgm){
			$this->execution_fgm = new file_group_execution($this->get_execution_files_filename(), $this->get_execution_files_directory());
		}
		return $this->execution_fgm;
	}

	function set_initial_file($name, $files){
		$filelist='';
		$basepath = $this->get_submission_directory();
		foreach($files as $file){
			$name = basename($file['name']);
			if($name>''){
				if($filelist>''){
					$filelist .=  "\n";
				}
				$filelist .= $name;
				$fp = vpl_fopen($basepath.$name);
				fwrite($fp,$file['data']);
				fclose($fp);
			}
		}
		$fp = vpl_fopen($this->get_submissionfilelistname());
		fwrite($fp,$filelist);
		fclose($fp);
	}

	/**
	 * get initial files
	 * @return array of array 'name' and 'data'
	 **/
	function get_initial_file($name){
		return array();
		$ret = array();
		$data =file_get_contents($this->get_submissionfilelistname());
		$nl = vpl_detect_newline($data);
		$filelist = explode("\n",$data);
		$basedir = $this->get_submission_directory();
		foreach ($filelist as $filename) {
			if($filename > ''){
				$data = file_get_contents($basedir.$filename);
				$ret[] = array('name' => $filename, 'data' => $data);
			}
		}
		return $ret;
	}

	/**
	 * set fulldescription
	 *
	 * @param $fulldescription string to save "with addslashes"
	 **/
	function set_fulldescription($fulldescription){
		$fp = vpl_fopen($this->get_fulldescription_filename());
		//remove form added addslashes
		$fulldescription = stripslashes_safe($fulldescription);
		fwrite($fp, $fulldescription);
		fclose($fp);
	}

	/**
	 * get fulldescription
	 *
	 * @return fulldescription
	 **/
	function get_fulldescription(){
		$filename = $this->get_fulldescription_filename();
		if(file_exists($filename)){
			return file_get_contents($filename);
		}
		else{
			return FALSE;
		}
	}

	/**
	 * Return maximum file size allowed
	 * @return int
	 **/
	function get_maxfilesize(){
		global $CFG;
		$max = vpl_get_max_post_size();
		if($CFG->vpl_maxfilesize > 0 && $CFG->vpl_maxfilesize < $max){
			$max = $CFG->vpl_maxfilesize;
		}
		if($this->instance->maxfilesize>0
		   && $this->instance->maxfilesize<$max){
			$max=$this->instance->maxfilesize;
		}
		return $max;
	}
	
	/**
	 * Get grading information help
	 * @return string grade comments summary in html format
	 **/
	function get_grading_help(){
		$list=array();
		$submissions=$this->all_last_user_submission();
		foreach($submissions as $submission){
			$sub = new mod_vpl_submission($this,$submission);
			$sub->filter_feedback($list);
		}
		//TODO show evaluation criteria with show hidde button
		$all= array();
		foreach($list as $text => $info){
			$astext = s(addslashes_js($text));
			$html='';
			$html.=s($text);
			foreach($info->grades as $grade=>$nothing){
				if($grade >=0){ //No grade
					$jscript='vpl_addComment(\''.$astext.'\')';
				}else{
					$jscript='vpl_addComment(\''.$astext.' ('.$grade.')\')';
				}
				$link='<a href="javascript:void(0)" onclick="'.$jscript.'">'.$grade.'</a>';
				$html.=' ('.$link.')';
			}
			$html.='<br />'."\n";
			if(isset($all[$info->count])){
				$all[$info->count] .= '('.$info->count.') '.$html;
			}else{
				$all[$info->count] = '('.$info->count.') '.$html;
			}
		}
		//Sort comments by number of occurrences
		krsort($all);
		$html='';
		foreach($all as $count => $info){
			$html.=$info;
		}
		//TODO show info about others review with show hidde button
		return $html;
	}
	/**
	 * Get password
	 **/
	function get_password(){
		return trim($this->instance->password);
	}

	/**
	 * Get password md5
	 **/
	function get_password_md5(){
		return md5($this->get_password().sesskey());
	}

	/**
	 * Check if pass password restriction
	 **/
	function pass_password_check(){
		$password = $this->get_password();
		if($password >''){
			global $SESSION;
			$password_md5 = $this->get_password_md5();
			$pasvar = 'vpl_password_'.$this->instance->id;
			if(isset($SESSION->$pasvar) && $SESSION->$pasvar == $password_md5){
				return true;
			}
			$pass_set=optional_param('password','',PARAM_TEXT);
			if($pass_set>''){
				if($pass_set == $password){
					$SESSION->$pasvar=$password_md5;
					return true;
				}
				sleep(5); //Wait 5 seg to limit force brute crack
			}
			return false;
		}
		return true;
	}

	/**
	 * Check password restriction
	 **/
	function password_check(){
		if(!$this->pass_password_check()){
			require_once('forms/password_form.php');
			$this->print_header();
			$mform = new mod_vpl_password_form($_SERVER['SCRIPT_NAME']);
			$mform->display();
			$this->print_footer();
			die;
		}
	}
	/**
	 * Check netword restriction
	 * Network field may be a combination of ip, netmask and domain
	 * conma separated to check.
	 **/
	function network_check(){
		$netreq = trim($this->instance->requirednet);
		if($netreq ==''){ // No net requered
			return;
		}
		$ip = getremoteaddr();
		$ipn=ip2long($ip);
		$nets = explode(',',$netreq); //separate elements (ip,  network, domain)
		foreach($nets as $net){
			$net=trim($net);
			if($net==''){ //No element
				continue;
			}
			if($net[0] >='0' && $net[0]<='9'){ //ip or network
				$parts=explode('/',$net);
				switch(count($parts)){
					case 1: //ip
						if($ip == $net){
							return;
						}
					break;
					case 2: //net mask
						$netip=ip2long($parts[0]);
						if(strpos($parts[1],'.')){//dot mask
							$netmask=ip2long($parts[1]);
						}else{ //number of bits mask
							$netmask=(0xffffffff)<<(32-$parts[1]);
						}
						if(($ipn & $netmask) == ($netip & $netmask)){
							return;
						}
				}
			}else{ //domain
				if($host=gethostbyaddr($ip)){
					$offset=strlen($host)-strlen($net);
					if($offset>=0 && substr($host,$offset)===$net){
						retun; //Found
					}
				}
			}
		}
		$this->print_header();
		print_box('<h2>'.get_string('opnotallowfromclient',VPL).'</h2>');
		$this->print_footer();
		die;
	}

	/**
	 * Check submission restriction
	 * @param $data Object with submited data
	 * @param & $error string
	 * @return bool
	 **/
	function pass_submission_restriction($alldata,& $error){
		$max = $this->get_maxfilesize();
		$rfn = $this->get_required_fgm();
		$list = $rfn->getFilelist();
		$error='';
		if(count($alldata)>$this->instance->maxfiles){
			$error.= get_string('maxfilesexceeded',VPL)."\n";
		}
		$lr=count($list);
		$lad=count($alldata);
		for($i=0; $i< $lad ; $i++){
			$name = $alldata[$i]['name'];
			$data = $alldata[$i]['data'];
			if(strlen($data)>$max){
				$error .= '"'.$name.'" '.get_string('maxfilesizeexceeded',VPL)."\n";
			}
			if($i<$lr && $list[$i] != $name){
				$error .= '"'.$name.'" '.get_string('incorrect_file_name',VPL)."\n";
			}
		}
		return strlen($error)==0;
	}

	/**
	 * Get user submissions, order reverse submission id
	 * @param $id user id
	 * @return FALSE/array of objects
	 **/
	function user_submissions($id){
		$select = "(userid = ".$id.") AND (vpl = ".$this->instance->id.")";
		return get_records_select('vpl_submissions', $select,'id DESC','*',0,1000);
	}

	/**
	 * Get all last usersubmission
	 * @return object array
	 **/
	function all_last_user_submission(){
		global $CFG;
		$tablename = $CFG->prefix.'vpl_submissions';
		//Get last submissions records for this vpl module
		$query = 'SELECT s.userid, s.* FROM '.$tablename.' s,';
		$query .= '(SELECT max(id) as maxid, userid, vpl FROM '.$tablename;
		$query .= ' WHERE vpl='.$this->get_instance()->id.' GROUP BY userid) ls';
		$query .= ' WHERE ls.maxid = s.id';
		return get_records_sql($query);
	}

	/**
	 * Get last usersubmission
	 * @param $id user id
	 * @return FALSE/object
	 **/
	function last_user_submission($id){
		$select = "(userid = ".$id.") AND (vpl = ".$this->instance->id.")";
		$res = get_records_select('vpl_submissions', $select,'id DESC','*',0,1);
		if($res){
			foreach($res as $sub){
				return $sub;
			}
		}
		return false;
	}

	static protected $context = array();
	/**
	 * Return context object for this module instance
	 * @return object
	 */
	public function get_context(){
		if(!isset($context[$this->cm->id])){
			$context[$this->cm->id] = get_context_instance(CONTEXT_MODULE, $this->cm->id);
		}
		return $context[$this->cm->id];
	}
	
	/**
	 * Requiere the current user has the capability of performing
	 * $capability in this module instance
	 * @param string $capability capability name
	 * @return void
	 */
	function require_capability($capability){
		require_capability($capability,$this->get_context());
	}
	
	/**
	 * Check if the user has the capability of performing
	 * $capability in this module instance
	 * @param string $capability capability name
	 * @param int $userid default NULL => current user
	 * @return bool
	 */
	function has_capability($capability,$userid=NULL){
		return has_capability($capability,$this->get_context(),$userid);
	}
	
	/**
	 * Delete overflow submissions.
	 * If three submissions within the period central is delete
	 * @param $id 
	 * @return void
	 **/
	function delete_overflow_submissions($id){
		//TODO add specific period to avery vpl
		global $CFG;
		if(!isset($CFG->vpl_discard_submission_period)){
			return;
		}
		if($CFG->vpl_discard_submission_period == 0){
			//Keep all submissions
			return;
		}
		if($CFG->vpl_discard_submission_period > 0){
			$select = "(userid = ".$id.") AND (vpl = ".$this->instance->id.")";
			$res = get_records_select('vpl_submissions', $select,'id DESC','*',0,3);
			if(count($res) == 3){
				$i=0;
				foreach($res as $sub){
					switch($i){
						case 0:$last=$sub;
						break;
						case 1:$second=$sub;
						break;
						case 2:$first=$sub;
						break;
					}
					$i++;
				}
				//Check time consistence
				if(!($last->datesubmitted > $second->datesubmitted &&
				 $second->datesubmitted > $first->datesubmitted)){
					return;
				}
				if(($last->datesubmitted - $first->datesubmitted)<$CFG->vpl_discard_submission_period){
					//Remove second submission
					$submission = new mod_vpl_submission($this,$second);
					$submission->delete();
				}
			}
		}
	}
	
	/**
	 * Check if it is submission period
	 * @return bool
	 **/
	function is_submission_period(){
		$now = time();
		$ret = $this->instance->availablefrom <= $now;
		return $ret && ($this->instance->duedate == 0 || $this->instance->duedate >= $now);
	}
	
	/**
	 * is visible this vpl instance
	 * @return bool
	 */
	function is_visible(){
		$cm = $this->get_course_module();
		$ret = true;
		$ret = $ret && $this->has_capability(VPL_VIEW_CAPABILITY);
		$ret = $ret && $this->get_instance()->visiblefrom < time();
		$ret = $ret && $cm->visible;
		//grader and manager always view
		$ret = $ret || $this->has_capability(VPL_GRADE_CAPABILITY);
		$ret = $ret || $this->has_capability(VPL_MANAGE_CAPABILITY);
		return $ret;
	}

	/**
	 * this vpl instance admit submission
	 * @return bool
	 */
	function is_submit_able(){
		$cm = $this->get_course_module();
		$instance = $this->get_instance();
		$ret = true;
		$ret = $ret && $this->has_capability(VPL_SUBMIT_CAPABILITY);
		$ret = $ret && $this->is_submission_period();
		$ret = $ret && $cm->visible;
		//manager can always submit
		$ret = $ret || $this->has_capability(VPL_MANAGE_CAPABILITY);
		return $ret;
	}

	/**
	 * Add log to the log system
	 *
	 * @param $action string (view, add, new, update, etc.)
	 * @param $url relative to module
	 * @param $info (optional) extra description
	 **/
	function add_to_log($action,$url,$info=''){
		$info = "{$this->instance->name} ".$info;
		if(strlen($info)>255){
			$info = substr($info,0,255);
		}
		add_to_log($this->get_course()->id,VPL,$action,$url,$info,
						$this->get_course_module()->id);
	}

	/**
	 * print end of page
	 **/
	function print_footer(){
		$style="position:absolute; z-index:2; right:10px; padding:8px; background-color: white;text-align:center;";
		echo '<div style="'.$style.'">';
		//print_box_start();
		echo '<a href="http://vpl.dis.ulpgc.es">';
		echo 'VPL 1.0b9';
		echo '</a>';
		//print_box_end();
		echo '</div>';
		print_footer($this->get_course());
	}

	/**
	 * print end of page
	 **/
	function print_footer_simple(){
		print_footer('empty');
	}
	
	/**
	 * print header
	 * @param $info title and last nav option
	 * @param $level int used for @see get_navigation_base
	 * @param $show_umb bool true and update module button
	 **/
	function print_header($info='',$show_umb=false, $body_atributtes=''){
		global $COURSE;
		$course = $this->get_course();
		$cm = $this->get_course_module();
		$strvpl = get_string('modulename', VPL);
		$tittle = $this->get_instance()->name;
		if($info){
			$tittle.=' '.$info;
		}
		//This FIX unknow error in mooodle when show view.php
		//***************************
		$COURSE = $course;
		//***************************
		$navigation = build_navigation($info,$cm);
		if($show_umb === true){
			$show_umb = update_module_button($cm->id, $course->id, $strvpl);
		}
		print_header($tittle, $course->fullname,$navigation,'', '', false, 
				$show_umb ,'',false,$body_atributtes);
	}
	function print_header_simple($body_atributtes=''){
		print_header('','','','','',false,'','',false,$body_atributtes);
	}
	
	/**
	 * Print heading action with help
	 *
	 * @param $action string base text and help
	 **/
	function print_heading_with_help($action){
		global $CFG;
		$icon = '<img src="'.$CFG->modpixpath.'/'.VPL.'/icon.gif" alt=""/>';
		$title = get_string($action,VPL).': '.$this->instance->name;
		print_heading_with_help($title, $action, VPL, $icon);
	}

	/**
	 * Remove end of tab code that add extra space after tab
	 * May be removed in the future
	 *
	 * @param $tabcode tabcode
	 * @return string
	 **/
	function remove_tabs_end($tabcode){
		$tabcode= trim($tabcode);
		return str_replace('</div><div class="clearer"> </div>', "</div>", $tabcode);
	}
	
	/**
	 * Create tabs to configure a vpl instance
	 *
	 * @param $id vpl instance
	 * @param $active tab
	 **/
	function print_configure_tabs($active){
		global $CFG;
		$active = basename($active);
		$strbasic = get_string('basic',VPL);
		$strfulldescription = get_string('fulldescription',VPL);
		$strfilerecuestes = get_string('requestedfiles',VPL);
		$strexecution = get_string('execution',VPL);
		$tabs= array();
		$tabs[]= new tabobject('edit',vpl_abs_href('/course/modedit.php','update',$this->cm->id),$strbasic,$strbasic);
		$tabs[]= new tabobject('fulldescription.php',vpl_mod_href('forms/fulldescription.php','id',$this->cm->id),$strfulldescription,$strfulldescription);
		$tabs[]= new tabobject('requestedfiles.php',vpl_mod_href('forms/requestedfiles.php','id',$this->cm->id),$strfilerecuestes,$strfilerecuestes);
		if($active=='executionfiles.php' || $active=='executionoptions.php' 
		   || $active=='executionlimits.php' || $active=='executionkeepfiles.php'){
			$tabs[]= new tabobject($active,vpl_mod_href('forms/executionfiles.php','id',$this->cm->id),$strexecution,$strexecution);
		   	$strexecutionfiles = get_string('executionfiles',VPL);
		   	$strexecutionoptions = get_string('executionoptions',VPL);
		   	$strexecutionlimits = get_string('resourcelimits',VPL);
		   	$strexecutionkeepfiles = get_string('keepfiles',VPL);
		   	$subtabs = array();
			$subtabs[]= new tabobject('executionfiles.php',vpl_mod_href('forms/executionfiles.php','id',$this->cm->id),$strexecutionfiles,$strexecutionfiles);
			$subtabs[]= new tabobject('executionoptions.php',vpl_mod_href('forms/executionoptions.php','id',$this->cm->id),$strexecutionoptions,$strexecutionoptions);
			$subtabs[]= new tabobject('executionlimits.php',vpl_mod_href('forms/executionlimits.php','id',$this->cm->id),$strexecutionlimits,$strexecutionlimits);
			$subtabs[]= new tabobject('executionkeepfiles.php',vpl_mod_href('forms/executionkeepfiles.php','id',$this->cm->id),$strexecutionkeepfiles,$strexecutionkeepfiles);
			echo $this->remove_tabs_end(print_tabs(array($tabs,$subtabs),$active,null,null,null, true));
		}
		else{
			$tabs[]= new tabobject('execution.php',vpl_mod_href('forms/executionfiles.php','id',$this->cm->id),$strexecution,$strexecution);
			echo $this->remove_tabs_end(print_tabs(array($tabs),$active, null,null, true));
		}
	}

	/**
	 * Create tabs to view_description/submit/view_submission/edition
	 *
	 * @param $active tab
	 **/
	function print_view_tabs($active){
		//TODO refactor using functions
		global $CFG;
		global $USER;
		$active = basename($active);
		$cmid=$this->cm->id;
		$userid = optional_param('userid',NULL,PARAM_INT);

		$viewer = $this->has_capability(VPL_VIEW_CAPABILITY);
		$submiter = $this->has_capability(VPL_SUBMIT_CAPABILITY);
		$similarity = $this->has_capability(VPL_SIMILARITY_CAPABILITY);
		$grader = $this->has_capability(VPL_GRADE_CAPABILITY);
		$manager = $this->has_capability(VPL_MANAGE_CAPABILITY);
		if(!$userid || ($userid && !$grader)){
			$userid = $USER->id;
		}
		$level2=$grader||$manager||$similarity;

		$strdescription = get_string('description',VPL);
		$strsubmission = get_string('submission',VPL);
		$stredition = get_string('edition',VPL);
		$strsubmissionview = get_string('submissionview',VPL);
		$maintabs = array();
		$tabs= array();
		$baseurl = $CFG->wwwroot.'/mod/'.VPL.'/';
		$href = vpl_mod_href('view.php','id',$cmid,'userid',$userid);
		$viewtab = new tabobject('view.php',$href,$strdescription,$strdescription);
		if($level2){
			if($viewer){
				$maintabs[]=$viewtab;
			}
			$strsubmissionslist = get_string('submissionslist',VPL);
			$href = vpl_mod_href('views/submissionslist.php','id',$cmid);
			$maintabs[]= new tabobject('submissionslist.php',$href,$strsubmissionslist,$strsubmissionslist);
			//similarity
			if($similarity){
				if($active == 'listwatermark.php' ||
					$active == 'similarity_form.php' ||
					$active == 'listsimilarity.php'){
					$tabname=$active;
				}else{
					$tabname = 'similarity';
				}
				$strsubmissionslist = get_string('similarity',VPL);
				$href = vpl_mod_href('similarity/similarity_form.php','id',$cmid);
				$maintabs[]= new tabobject($tabname,$href,$strsubmissionslist,$strsubmissionslist);
			}
			//test
			if($grader||$manager){
				if($userid == $USER->id){
					$text = get_string('test',VPL);
				}else{
					$user = get_record('user','id',$userid);
					$text = get_string('user').' '.fullname($user);
				}
				if($active == 'submission.php' ||
					$active == 'edition.php' ||
					$active == 'submissionview.php' ||
					$active == 'gradesubmission.php' ||
					$active == 'previoussubmissionslist.php'){
					$tabname=$active;
				}else{
					$tabname = 'test';
				}
				$href = vpl_mod_href('forms/submissionview.php','id',$cmid,'userid',$userid);
				$maintabs[]= new tabobject($tabname,$href,$text,$text);
			}
		}
		switch($active){
			case 'view.php':
				if($level2){
					echo $this->remove_tabs_end(print_tabs(array($maintabs,$tabs),$active,null,null,true));
					return;
				}
			case 'submission.php':
			case 'edition.php':
			case 'submissionview.php':
			case 'gradesubmission.php':
			case 'previoussubmissionslist.php':
				require_once('vpl_submission.class.php');
				$subinstance = $this->last_user_submission($userid);
				if($subinstance !== false){
					$submission = new mod_vpl_submission($this,$subinstance);
				}
				if($viewer && ! $level2){
					$tabs[] = $viewtab; 
				}
				if($manager || ($grader && $USER->id == $userid)
				 || (!$grader && $submiter && $this->is_submit_able() && !$this->instance->restrictededitor )){
					$href = vpl_mod_href('forms/submission.php','id',$cmid,'userid',$userid);
					$tabs[]= new tabobject('submission.php',$href,$strsubmission,$strsubmission);
				}
				if($manager || ($grader && $USER->id == $userid) || (!$grader && $submiter && $this->is_submit_able())){
					$href = vpl_mod_href('forms/edition.php','id',$cmid,'userid',$userid);
					$tabs[]= new tabobject('edition.php',$href,$stredition,$stredition);
				}
				if( $subinstance !== false) {
					$href = vpl_mod_href('forms/submissionview.php','id',$cmid,'userid',$userid);
					$tabs[]= new tabobject('submissionview.php',$href,$strsubmissionview,$strsubmissionview);
				}
				if($grader && $this->instance->grade !=0 &&$subinstance
				   && ($subinstance->dategraded==0 || $subinstance->grader==$USER->id)){
					$href = vpl_mod_href('forms/gradesubmission.php','id',$cmid,'userid',$userid);
					$text=get_string('grade');
					$tabs[]= new tabobject('gradesubmission.php',$href,$text,$text);
				}
				if($subinstance && ($grader || $similarity) ){
					$strlistprevoiussubmissions = get_string('previoussubmissionslist',VPL);
					$href = vpl_mod_href('views/previoussubmissionslist.php','id',$cmid,'userid',$userid);
					$tabs[]= new tabobject('previoussubmissionslist.php',$href,$strlistprevoiussubmissions,$strlistprevoiussubmissions);
				}
				if($level2){
					if(isset($user)){
						echo '<div style="position:absolute; z-index:5;">';
						print_user_picture($user,$this->course->id);
						echo '</div>';
					}
					echo $this->remove_tabs_end(print_tabs(array($maintabs,$tabs),$active,null,null,true));
					return;
				}
				else{
					echo $this->remove_tabs_end(print_tabs(array($tabs),$active,null,null,true));
					return;
				}
				
			break;
			case 'submissionslist.php':
				echo $this->remove_tabs_end(print_tabs(array($maintabs),$active,null,null,true));
			return;
			case 'listwatermark.php':
			case 'similarity_form.php':
			case 'listsimilarity.php':
				if( $similarity ){
					$href = vpl_mod_href('similarity/similarity_form.php','id',$cmid);
					$string=get_string('similarity',VPL);
					$tabs[]= new tabobject('similarity_form.php',$href,$string,$string);
					if($active == 'listsimilarity.php'){
						$string=get_string('listsimilarity',VPL);
						$tabs[]= new tabobject('listsimilarity.php','',$string,$string);
					}
					$href = vpl_mod_href('similarity/listwatermark.php','id',$cmid);
					$string=get_string('listwatermarks',VPL);
					$tabs[]= new tabobject('listwatermark.php',$href,$string,$string);
				}
				echo $this->remove_tabs_end(print_tabs(array($maintabs,$tabs),$active,null,null,true));
			break;
		}
	}

	/**
	 * Show vpl name
	 **/
	function print_name(){
		echo '<h2>';
		print($this->instance->name);
		echo '</h2>';
	}

	/**
	 * Show vpl submission period
	 **/
	function print_submission_period(){
		if($this->instance->visiblefrom){
			echo '<b>';
			echo get_string('visiblefrom',VPL);
			echo '</b> ';
			echo userdate($this->instance->visiblefrom);
			echo '<br />';
		}
		if($this->instance->availablefrom){
			echo '<b>';
			echo get_string('availablefrom',VPL);
			echo '</b> ';
			echo userdate($this->instance->availablefrom);
			echo '<br />';
		}
		if($this->instance->duedate){
			echo '<b>';
			echo get_string('duedate',VPL);
			echo '</b> ';
			echo userdate($this->instance->duedate);
			echo '<br />';
		}
	}

	/**
	 * Show vpl submission restriction
	 **/
	function print_submission_restriction(){
		global $modname;
		//TODO print_submission_restriction
	}

	/**
	 * Show short description
	 **/
	function print_shordescription(){
		if($this->instance->shortdescription){
			print_box_start();
			echo format_text($this->instance->shortdescription,FORMAT_HTML);
			print_box_end();
		}
	}

	/**
	 * Show full description
	 **/
	function print_fulldescription(){
		$full = $this->get_fulldescription();
		if($full){
			print_box_start();
			echo format_text($full,FORMAT_HTML);
			print_box_end();
		}
		else{
			$this->print_shordescription();
		}
	}
}

?>